#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

void use_inet_addr(const char *name);
void use_inet_aton(const char *name);
void use_inet_pton(const char *name, int af);
void use_gethostbyname(const char *name);
void use_gethostbyname_r(const char *name);
void use_gethostbyname2(const char *name, int af);
void use_gethostbyname2_r(const char *name, int af);
void use_getaddrinfo(const char *name);

int main(int argc, char *argv[])
{
	if(argc != 2) {
		printf("Usage: %s <IPv4 addr>\n", argv[0]);
		return 1;
	}

	use_inet_addr(argv[1]);
	use_inet_aton(argv[1]);
	use_inet_pton(argv[1], AF_INET);
	use_gethostbyname(argv[1]);
	use_gethostbyname_r(argv[1]);
#if defined(_BSD_SOURCE) || defined(_SVID_SOURCE)
	use_gethostbyname2(argv[1], AF_INET);
	use_gethostbyname2_r(argv[1], AF_INET);
#endif
	use_getaddrinfo(argv[1]);

	return 0;
}

void use_inet_addr(const char *name)
{
	in_addr_t addr = inet_addr(name);

	printf("inet_addr: ");
	if(addr == INADDR_NONE) {
		printf("failed (possibly 255.255.255.255)\n");
	} else {
		struct in_addr in;
		in.s_addr = addr;
		printf("%s\n", inet_ntoa(in));
	}
}

void use_inet_aton(const char *name)
{
	struct in_addr in;

	printf("inet_aton: ");
	if(inet_aton(name, &in)) {
		printf("%s\n", inet_ntoa(in));
	} else {
		printf("failed\n");
	}
}

void use_inet_pton(const char *name, int af)
{
	struct in_addr in;
	
	printf("inet_pton: ");
	if(inet_pton(af, name, &in)) {
		printf("%s\n", inet_ntoa(in));
	} else {
		printf("failed\n");
	}
}

void use_gethostbyname(const char *name)
{
	struct hostent *h = gethostbyname(name);

	printf("gethostbyname: ");
	if(!h) {
		printf("failed (%s)\n", hstrerror(h_errno));
	} else {
		if(h->h_addrtype != AF_INET) {
			printf("failed (invalid address type)\n");
		} else {
			char **pp = h->h_addr_list;
			while(*pp != NULL) {
				struct in_addr *p = (struct in_addr*)(*pp);
				printf("%s\n", inet_ntoa(*p));
				++pp;
			}
		}
	}
}

void use_gethostbyname_r(const char *name)
{
	int rc;
	char buf[8192];
	struct hostent h;
	struct hostent *rp;
	int myerrno;

	printf("gethostbyname_r: ");
	rc = gethostbyname_r(name, &h, buf, sizeof(buf), &rp, &myerrno);
	if(rc == ERANGE) {
		printf("failed (out of memory)\n");
	} else if(!rc) {
		if(!rp) {
			printf("no address found\n");
		} else {
			char **pp = h.h_addr_list;
			while(*pp != NULL) {
				struct in_addr *p = (struct in_addr*)(*pp);
				printf("%s\n", inet_ntoa(*p));
				++pp;
			}
		}
	} else {
		printf("failed (%s)\n", hstrerror(myerrno));
	}
}

#if defined(_BSD_SOURCE) || defined(_SVID_SOURCE)

void use_gethostbyname2(const char *name, int af)
{
	struct hostent *h = gethostbyname2(name, af);

	printf("gethostbyname2: ");
	if(!h) {
		printf("failed (%s)\n", hstrerror(h_errno));
	} else {
		if(h->h_addrtype != AF_INET) {
			printf("failed (invalid address type)\n");
		} else {
			char **pp = h->h_addr_list;
			while(*pp != NULL) {
				struct in_addr *p = (struct in_addr*)(*pp);
				printf("%s\n", inet_ntoa(*p));
				++pp;
			}
		}
	}
}

void use_gethostbyname2_r(const char *name, int af)
{
	int rc;
	char buf[8192];
	struct hostent h;
	struct hostent *rp;
	int myerrno;

	printf("gethostbyname2_r: ");
	rc = gethostbyname2_r(name, af, &h, buf, sizeof(buf), &rp, &myerrno);
	if(rc == ERANGE) {
		printf("failed (out of memory)\n");
	} else if(!rc) {
		if(!rp) {
			printf("no address found\n");
		} else {
			char **pp = h.h_addr_list;
			while(*pp != NULL) {
				struct in_addr *p = (struct in_addr*)(*pp);
				printf("%s\n", inet_ntoa(*p));
				++pp;
			}
		}
	} else {
		printf("failed (%s)\n", hstrerror(myerrno));
	}
}

#endif

void use_getaddrinfo(const char *name)
{
	int rc;
	struct addrinfo hints;
	struct addrinfo *resp;

	printf("getaddrinfo: ");

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_flags = AI_ADDRCONFIG | AI_PASSIVE;
	hints.ai_socktype = SOCK_STREAM;
	
	rc = getaddrinfo(name, NULL, &hints, &resp);
	if(rc) {
		printf("failed (%s)\n", gai_strerror(rc));
	} else {
		struct addrinfo *rp;
		
		for(rp = resp; rp != NULL; rp = rp->ai_next) {
			struct sockaddr_in *addr = (struct sockaddr_in*)(rp->ai_addr);
			printf("%s\n", inet_ntoa(addr->sin_addr));
		}

		freeaddrinfo(resp);
	}
}

